//
//  MainController.m
//  RollOverButtonTest
//
//  Created by Andreas on Fri Jan 30 2004.
//  Copyright (c) 2004 Andreas Mayer. All rights reserved.
//
//	2011-08-08	This is now a custom button editor instead of just a test application
//					- renamed some outlets
//					- added -importDesign: and -exportDesign: methods; renamed -revertColors: to -revertDesign:
//					- removed -click: and -createMatrix methods


#import "MainController.h"
#import "AMRollOverButtonCell.h"


@interface AMRollOverButtonCell (MainController)
- (void)finishInit;
@end


// ======================================================
#pragma mark - Class Extension
// ======================================================

@interface MainController ()
- (NSColor *)randomColor;
- (NSShadow *)textShadowWithColor:(NSColor *)color;
- (NSColor *)colorForShadow:(NSShadow *)shadow;
- (void)updateColors;
- (void)updatePopUps;
- (void)sizeToFit;
@end


// ======================================================
#pragma mark - Implementation
// ======================================================

@implementation MainController

@synthesize defaultAppearance;
@synthesize clickSound;

// ======================================================
#pragma mark - Initialization
// ======================================================

- (void)awakeFromNib
{
	if (!theWindow) {
		[NSBundle loadNibNamed:@"ButtonEditor" owner:self];
		[viewThatHoldsStupidVerticalSpacingConstraint removeConstraint:stupidVerticalSpacingConstraint];
		// set up action for roll over button
		// (anyone knows how to set up a self made IB palette with custom actions?)
		[theButton setAction:@selector(click:)];
		[theButton setTarget:self];
		[theButton setAlignment:NSCenterTextAlignment];
		[theButton setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:[theButton controlSize]]]];

		[[NSColorPanel sharedColorPanel] setShowsAlpha:YES];
		
		[self updateColors];
		[self updatePopUps];
		[self changeMenuDelay:nil]; // update display
		[self setDefaultAppearance:[[[[theButton class] appearance] copy] autorelease]];
//		[self createMatrix];
		[theWindow makeKeyAndOrderFront:self];
	}
}

- (void)dealloc {
    [defaultAppearance release];
	[clickSound release];
    [super dealloc];
}

// ======================================================
#pragma mark - Action Methods
// ======================================================

- (IBAction)ImportDesign:(id)sender {
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	[openPanel setTitle:@"OPEN PANEL - EXPORT DESIGN"];
	[openPanel setCanChooseFiles:YES];
	[openPanel setCanChooseDirectories:NO];
	[openPanel setAllowsMultipleSelection:NO];
	[openPanel beginSheetModalForWindow:theWindow completionHandler:
	 ^(NSInteger button){
		 if (button == NSFileHandlingPanelOKButton) {
			 NSURL *fileURL = [openPanel URL];
			 NSError *theError = nil;
			 NSData *buttonData = [NSData dataWithContentsOfURL:fileURL options:0 error:&theError];
			 if (theError) {
				 [[NSApplication sharedApplication] presentError:theError];
			 } else {
				 AMRollOverButtonAppearance *newAppearance = [AMRollOverButtonAppearance appearanceWithDataRepresentation:buttonData];
				 [[theButton class] setAppearance:newAppearance];
				 [[theButton cell] invalidateLayout];
				 [theButton setNeedsDisplay];
				 // adjust UI
				 [self updateColors];
				 [self updatePopUps];
			 }
		 }
	 }
	 ];
}

- (IBAction)ExportDesign:(id)sender {
	NSSavePanel *savePanel = [NSSavePanel savePanel];
	[savePanel setTitle:@"SAVE PANEL - EXPORT DESIGN"];
	[savePanel beginSheetModalForWindow:theWindow completionHandler:
	 ^(NSInteger button) {
		 if (button == NSFileHandlingPanelOKButton) {
			 AMRollOverButtonAppearance *theAppearance = [[theButton class] appearance];
			 NSData *buttonData = [AMRollOverButtonAppearance dataRepresentationFromAppearance:theAppearance];
			 NSURL *fileURL = [savePanel URL];
			 NSError *theError = nil;
			 if (![buttonData writeToURL:fileURL options:NSDataWritingAtomic error:&theError]) {
				 [[NSApplication sharedApplication] presentError:theError];
			 }
		 }
	 }
	];
}

- (IBAction)changeColor:(id)sender
{
	if (sender == controlColor) {
			[theButton setControlColor:[sender color]];
	} else if (sender == shadingColor) {
		if ([[sender color] alphaComponent] < 0.0001) {
			[theButton setShadingColor:nil];
		} else {
			[theButton setShadingColor:[sender color]];
		}
	} else if (sender == frameColor) {
		[theButton setFrameColor:[sender color]];
	} else if (sender == textColor) {
		[theButton setTextColor:[sender color]];
	} else if (sender == arrowColor) {
		[theButton setArrowColor:[sender color]];
	} else if (sender == textShadowColor) {
		[theButton setTextShadow:[self textShadowWithColor:[sender color]]];
	} else if (sender == mouseoverControlColor) {
		[theButton setMouseoverControlColor:[sender color]];
	} else if (sender == mouseoverShadingColor) {
		if ([[sender color] alphaComponent] < 0.0001) {
			[theButton setMouseoverShadingColor:nil];
		} else {
			[theButton setMouseoverShadingColor:[sender color]];
		}
	} else if (sender == mouseoverFrameColor) {
		[theButton setMouseoverFrameColor:[sender color]];
	} else if (sender == mouseoverTextColor) {
		[theButton setMouseoverTextColor:[sender color]];
	} else if (sender == mouseoverArrowColor) {
		[theButton setMouseoverArrowColor:[sender color]];
	} else if (sender == mouseoverTextShadowColor) {
		[theButton setMouseoverTextShadow:[self textShadowWithColor:[sender color]]];
	} else if (sender == highlightedControlColor) {
		[theButton setHighlightedControlColor:[sender color]];
	} else if (sender == highlightedShadingColor) {
		if ([[sender color] alphaComponent] < 0.0001) {
			[theButton setHighlightedShadingColor:nil];
		} else {
			[theButton setHighlightedShadingColor:[sender color]];
		}
	} else if (sender == highlightedFrameColor) {
		[theButton setHighlightedFrameColor:[sender color]];
	} else if (sender == highlightedTextColor) {
		[theButton setHighlightedTextColor:[sender color]];
	} else if (sender == highlightedArrowColor) {
		[theButton setHighlightedArrowColor:[sender color]];
	} else if (sender == highlightedTextShadowColor) {
		[theButton setHighlightedTextShadow:[self textShadowWithColor:[sender color]]];
	} else if (sender == controlShadowColor) {
		[theButton setControlShadow:[self textShadowWithColor:[sender color]]];
	}
	[theButton setNeedsDisplay:YES];
}

- (IBAction)changeTitle:(id)sender
{
	[theButton setTitle:[sender stringValue]];
	[self sizeToFit];
	[theButton setNeedsDisplay:YES];
}

- (IBAction)changeStyle:(id)sender
{
	NSRect oldFrame = [theButton frame];
	[theButton setBezelStyle:[sender selectedTag]];
	[theButton sizeToFit];
	[[theButton superview] setNeedsDisplayInRect:oldFrame];
	[theButton setNeedsDisplay:YES];
}

- (IBAction)changeSize:(id)sender
{
	NSRect oldFrame = [theButton frame];
	[theButton setControlSize:[[sizePopUp selectedItem] tag]];
	NSFont *newFont = [[NSFontManager sharedFontManager] convertFont:[theButton font] toSize:[NSFont systemFontSizeForControlSize:[theButton controlSize]]];
	[theButton setFont:newFont];
	[theButton sizeToFit];
	[[theButton superview] setNeedsDisplayInRect:oldFrame];
	[theButton setNeedsDisplay:YES];
}

- (IBAction)changeDisplayBorder:(id)sender
{
	[theButton setBordered:([displayBorder state] == NSOnState)];
	[theButton setNeedsDisplay:YES];
}

- (IBAction)changeDisplayMenu:(id)sender
{
	if ([sender state] == NSOnState) {
		[[theButton cell] setMenu:theMenu];
	} else {
		[[theButton cell] setMenu:nil];
	}
	[self sizeToFit];
	[theButton setNeedsDisplay:YES];
}

- (IBAction)changeDisplayArrow:(id)sender
{
	[(AMRollOverButtonCell *)[theButton cell] setAlwaysShowMenuIndicator:([sender state] == NSOnState)];
	[theButton sizeToFit];
	[theButton setNeedsDisplay:YES];
}

- (IBAction)changeButtonType:(id)sender
{
	NSButtonType newType = [sender selectedTag];
	if (newType == NSOnOffButton) {
		[theButton setButtonType:NSOnOffButton];
	} else {
		[theButton setButtonType:NSMomentaryPushButton];
	}
	[theButton setNeedsDisplay:YES];
}

- (IBAction)revertDesign:(id)sender
{
	[[theButton class] setAppearance:[[defaultAppearance copy] autorelease]];
	[[theButton cell] invalidateLayout];
	[theButton setNeedsDisplay:YES];
	[self updateColors];
	[self updatePopUps];
}

- (IBAction)changeMenuDelay:(id)sender
{
	float menuDelay = [(AMRollOverButtonCell *)[theButton cell] popUpMenuDelay];
	if (sender) {
		menuDelay = [sender floatValue];
		[[theButton cell] setPopUpMenuDelay:menuDelay];
	}
	[menuDelayText setFloatValue:menuDelay];
	[menuDelaySlider setFloatValue:menuDelay];
}

- (IBAction)changeShadingMode:(id)sender
{
	[theButton setShadingMode:[sender selectedTag]];
	[theButton setNeedsDisplay:YES];
}

- (IBAction)click:(id)sender
{
	[clickSound play];
}


// ======================================================
#pragma mark - Utility Methods
// ======================================================

- (NSColor *)randomColor
{
	float red = (random()%1000)/1000.0;
	float green = (random()%1000)/1000.0;
	float blue = (random()%1000)/1000.0;
	float alpha = (random()%1000)/1000.0;
	return [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:alpha];
}

- (NSShadow *)textShadowWithColor:(NSColor *)color
{
	NSShadow *result = nil;
	if ([color alphaComponent] != 0.0) {
		result = [[[NSShadow alloc] init] autorelease];
		[result setShadowOffset:NSMakeSize(0.0, -1.0)];
		[result setShadowBlurRadius:1.0];
		[result setShadowColor:color];
	}
	return result;
}

- (NSColor *)colorForShadow:(NSShadow *)shadow
{
	NSColor *result;
	if (shadow != nil) {
		result = [shadow shadowColor];
	} else {
		result = [NSColor clearColor];
	}
	return result;
}

- (void)updateColors
{
	NSColor *someColor = nil;
	NSColor *clearColor = [NSColor clearColor];
	[controlColor setColor:[theButton controlColor]];
	someColor = [theButton shadingColor];
	[shadingColor setColor:(someColor ? someColor : clearColor)];
	[frameColor setColor:[theButton frameColor]];
	[textColor setColor:[theButton textColor]];
	[arrowColor setColor:[theButton arrowColor]];
	[textShadowColor setColor:[self colorForShadow:[theButton textShadow]]];
	[controlShadowColor setColor:[self colorForShadow:[theButton controlShadow]]];
	[mouseoverControlColor setColor:[theButton mouseoverControlColor]];
	someColor = [theButton mouseoverShadingColor];
	[mouseoverShadingColor setColor:(someColor ? someColor : clearColor)];
	[mouseoverFrameColor setColor:[theButton mouseoverFrameColor]];
	[mouseoverTextColor setColor:[theButton mouseoverTextColor]];
	[mouseoverArrowColor setColor:[theButton mouseoverArrowColor]];
	[mouseoverTextShadowColor setColor:[self colorForShadow:[theButton mouseoverTextShadow]]];
	[highlightedControlColor setColor:[theButton highlightedControlColor]];
	someColor = [theButton highlightedShadingColor];
	[highlightedShadingColor setColor:(someColor ? someColor : clearColor)];
	[highlightedFrameColor setColor:[theButton highlightedFrameColor]];
	[highlightedTextColor setColor:[theButton highlightedTextColor]];
	[highlightedArrowColor setColor:[theButton highlightedArrowColor]];
	[highlightedTextShadowColor setColor:[self colorForShadow:[theButton highlightedTextShadow]]];
}

- (void)updatePopUps
{
	[typePopUp selectItemWithTag:[theButton buttonType]];
	[stylePopUp selectItemWithTag:[[theButton cell] amBezelStyle]];
	[shadingPopUp selectItemWithTag:[[theButton cell] shadingMode]];
	BOOL flag = [theButton isBordered];
	[displayBorder setState:(flag ? NSOnState : NSOffState)];
}

- (void)sizeToFit
{
	[[theButton superview] setNeedsDisplayInRect:[theButton frame]];
	[theButton sizeToFit];
}

// ======================================================
#pragma mark - Application Delegate Methods
// ======================================================

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
	// initialize button click sound
	self.clickSound = [NSSound soundNamed:@"click"];
	self.clickSound.volume = 0.0;
	[self.clickSound play];
	[self.clickSound stop];
	self.clickSound.volume = 1.0;
}


@end
